from lxml import etree
import ntriples
import StringIO
import simplejson

SPARQLNS = u'http://www.w3.org/2005/sparql-results#'

def parse_sparql_result(xml):
    doc = etree.fromstring(xml)
    results = []
    for result in doc.xpath('/s:sparql/s:results/s:result',
                            namespaces={'s': SPARQLNS}):
        data = {}
        for binding in result:
            name = unicode(binding.attrib['name'])
            for value in binding:
                type = unicode(value.tag.split('}')[-1])
                lang = value.attrib.get(
                    '{http://www.w3.org/XML/1998/namespace}lang')
                datatype = value.attrib.get('datatype')
                text = value.text
                if not text is None and not isinstance(text, unicode):
                    text = text.decode('utf8')

                if type == 'uri':
                    # allegro graph returns context uri's with <> chars
                    if text.startswith('<'):
                        text = text[1:]
                    if text.endswith('>'):
                        text = text[:-1]
                    
                data[name] = {'value': text,
                              'type': type}
                if not lang is None:
                    # w3c sparql json result spec says 'xml:lang',
                    # we use 'lang' instead, just like the dict serialization
                    data[name]['lang'] = lang.decode('utf8')
                if not datatype is None:
                    data[name]['datatype'] = datatype.decode('utf8')
                    # w3c sparql json result spec says 'type' should not be
                    # 'literal', but 'typed-literal', we don't do this
                    
                    #data[name]['type'] = 'typed-literal'

        results.append(data)
    if not results:
        # maybe this is an ASK query response
        bools = doc.xpath('/s:sparql/s:boolean/text()',
                           namespaces={'s': SPARQLNS})
        if bools:
            if bools[0] == 'true':
                return True
            else:
                return False
    return results

def ntriples_to_dict(file):

    class TripleDict(dict):
        def triple(self, s, p, o):
            if isinstance(s, ntriples.URI):
                subject = unicode(s)
            elif isinstance(s, ntriples.bNode):
                subject = u'_:%s' % s 
            else:
                raise ValueError('Unknown subject type: %s' % type(s)) 
            predicate = unicode(p)
            predicates = self.get(subject, {})
            values = predicates.get(predicate, [])
            value = {'value' : unicode(o)}
            if isinstance(o, ntriples.URI):
                value['type'] = u'uri'
            elif isinstance(o, ntriples.bNode):
                value['type'] = u'bnode'
            elif isinstance(o, ntriples.Literal):
                lang, dtype, literal = o.split(' ', 2)
                value['type'] = u'literal'
                value['value'] = literal
                if lang != 'None':
                    value['lang'] = lang
                elif dtype != 'None':
                    value['datatype'] = dtype
            else:
                raise ValueError('Unknown object type: %s' % type(o)) 

            values.append(value)
            self[subject] = predicates
            predicates[predicate] = values
            
    parser = ntriples.NTriplesParser(TripleDict())
    result = parser.parse(file)
    return dict(result)

def dict_to_ntriples(data):
    result = StringIO.StringIO()
    for subject, predicates in data.items():
        if not subject.startswith('_:'):
            subject = u'<%s>' % subject
        for predicate, values in predicates.items():
            predicate = u'<%s>' % predicate
            for value in values:
                if value['type'] == 'uri':
                    object = '<%s>' % value['value']
                elif value['type'] == 'bnode':
                    object = '_:%s' % value['value']
                else:
                    object = value['value']
                    object = object.replace('\\', '\\\\')
                    object = object.replace('\t', '\\t')
                    object = object.replace('\n', '\\n')
                    object = object.replace('\r', '\\r')
                    object = object.replace('\"', '\\"')
                    object = '"%s"' % object
                    if value.get('lang'):
                        object = u'%s@%s' % (object, value['lang'])
                    elif value.get('datatype'):
                        object = u'%s^^<%s>' % (object, value['datatype'])
                ntriple = '%s %s %s .\n' % (subject, predicate, object)
                result.write(ntriple)
    result.seek(0)
    return result

def json_to_ntriples(data):
    data = simplejson.load(data)
    return dict_to_ntriples(data)
    
def ntriples_to_json(triples):
    data = ntriples_to_dict(triples)
    return StringIO.StringIO(simplejson.dumps(data, indent=True))



from zope.interface import Interface, Attribute

class ITripleStore(Interface):

    def connect(uri):
        """
        Connect to database, retrieve an ITripleStore instance
        """

    def disconnect():
        """
        Disconnect the database
        """
        
    def contexts():
        """
        Return a list of context names
        """
        
    def clear(context_name):
        """
        Remove all triples from a context
        """

    def register_prefix(prefix, namespace):
        """
        Register a namespace with a specific prefix
        These will be used in serializing rdfxml
        if backend supports this
        """
        
    def count(context_name=None):
        """
        Return the number of triples in the database or
        None if the backend does not support this
        Optionally a context_name can be specified
        """
        
    def add_rdfxml(uri_string_or_file, context_name, base_uri):
        """
        Add triples data in rdfxml format to a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def add_ntriples(uri_string_or_file, context_name):
        """
        Add triples data in ntriples format to a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def add_turtle(uri_string_or_file, context_name):
        """
        Add triples data in turtle format to a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def add_json(uri_string_or_file, context_name):
        """
        Add triples data in json format to a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def add_dict(dictionary, context_name):
        """
        Add triples data as a python dictionary to a specific context
        """

    def remove_rdfxml(uri_string_or_file, context_name, base_uri):
        """
        Remove triples data in rdfxml format from a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def remove_ntriples(uri_string_or_file, context_name):
        """
        Remove triples data in ntriples format from a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def remove_turtle(uri_string_or_file, context_name):
        """
        Remove triples data in turtle format from a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
    def remove_json(uri_string_or_file, context_name):
        """
        Remove triples data in json format from a specific context
        
        data can be either a URI string starting with 'http://' or 'file://',
        it can also be a string containing the data, or a file like
        object with a read, seek, and close method.      
        """
        
    def remove_dict(dictionary, context_name):
        """
        Remove triples data as a python dictionary from a specific context
        """
        
    def get_rdfxml(context_name, pretty=False):
        """
        Returns a file object (something with a read and close method)
        containing the triple data from a specific context in rdfxml format
        """
        
    def get_ntriples(context_name):
        """
        Returns a file object (something with a read and close method)
        containing the triple data from a specific context in ntriples format
        """
        
    def get_turtle(context_name):
        """
        Returns a file object (something with a read and close method)
        containing the triple data from a specific context in turtle format
        """
        
    def get_json(context_name):
        """
        Returns a file object (something with a read and close method)
        containing the triple data from a specific context in json format
        """

    def get_dict(context_name):
        """
        Returns a python dictionary containing the triple data
        from a specific context
        """

class ISPARQLEndpoint(Interface):
    def select(sparql_query):
        """
        Run a sparql SELECT query, returns a list
        of dictionaries in sparql result format (json-like)
        """

    def ask(sparql_query):
        """
        Run a sparql ASK query, returns a boolean
        """
    def construct(sparql_query, format):
        """
        Run a sparql CONSTRUCT query, returns a
        filestream with triples in the specifed format
        or None if no result was found
        """
        



#!/usr/bin/env python
"""
N-Triples Parser
Copyright 2004, Sean B. Palmer, inamidst.com
Licensed under GPL 2, W3C, BSD, MIT, or EFL 2
Documentation: 
   http://inamidst.com/proj/rdf/ntriples-doc

Command line usage: 
   ./ntriples.py <URI>    - parses URI as N-Triples
   ./ntriples.py --help   - prints out this help message
# @@ fully empty document?
"""

import re

uriref = r'<([^:]+:[^\s"<>]+)>'
literal = r'"([^"\\]*(?:\\.[^"\\]*)*)"'
litinfo = r'(?:@([a-z]+(?:-[a-z0-9]+)*)|\^\^' + uriref + r')?'

r_line = re.compile(r'([^\r\n]*)(?:\r\n|\r|\n)')
r_wspace = re.compile(r'[ \t]*')
r_wspaces = re.compile(r'[ \t]+')
r_tail = re.compile(r'[ \t]*\.[ \t]*')
r_uriref = re.compile(uriref)
r_nodeid = re.compile(r'_:([A-Za-z][A-Za-z0-9]*)')
r_literal = re.compile(literal + litinfo)

bufsiz = 2048
validate = False

class Node(unicode): pass

class URI(Node): pass
class bNode(Node): pass
class Literal(Node):
   def __new__(cls, lit, lang=None, dtype=None):
      n = str(lang) + ' ' + str(dtype) + ' ' + lit
      return unicode.__new__(cls, n)

class Sink(object): 
   def __init__(self): 
      self.length = 0

   def triple(self, s, p, o): 
      self.length += 1
      print (s, p, o)

class ParseError(Exception): pass

quot = {'t': '\t', 'n': '\n', 'r': '\r', '"': '"', '\\': '\\'}
r_safe = re.compile(r'([\x20\x21\x23-\x5B\x5D-\x7E]+)')
r_quot = re.compile(r'\\(t|n|r|"|\\)')
r_uniquot = re.compile(r'\\u([0-9A-F]{4})|\\U([0-9A-F]{8})')

def unquote(s): 
   """Unquote an N-Triples string."""
   result = []
   
   while s: 
      m = r_safe.match(s)
      if m: 
         s = s[m.end():]
         result.append(m.group(1))
         continue

      m = r_quot.match(s)
      if m: 
         s = s[2:]
         result.append(quot[m.group(1)])
         continue 

      m = r_uniquot.match(s)
      if m: 
         s = s[m.end():]
         u, U = m.groups()
         codepoint = int(u or U, 16)
         if codepoint > 0x10FFFF: 
            raise ParseError("Disallowed codepoint: %08X" % codepoint)
         result.append(unichr(codepoint))
      elif s.startswith('\\'): 
         raise ParseError("Illegal escape at: %s..." % s[:10])
      else: raise ParseError("Illegal literal character: %r" % s[0])
   return unicode(''.join(result))

if not validate: 
   def unquote(s): 
      return s.decode('unicode-escape')

r_hibyte = re.compile(r'([\x80-\xFF])')

def uriquote(uri): 
   return r_hibyte.sub(lambda m: '%%%02X' % ord(m.group(1)), uri)
if not validate: 
   def uriquote(uri): 
      return uri

class NTriplesParser(object): 
   """An N-Triples Parser.
      Usage: 
         p = NTriplesParser(sink=MySink())
         sink = p.parse(f) # file; use parsestring for a string
   """

   def __init__(self, sink=None): 
      if sink is not None: 
         self.sink = sink
      else: self.sink = Sink()

   def parse(self, f): 
      """Parse f as an N-Triples file."""
      if not hasattr(f, 'read'): 
         raise ParseError("Item to parse must be a file-like object.")

      self.file = f
      self.buffer = ''
      while True: 
         self.line = self.readline()
         if self.line is None: break
         try: self.parseline()
         except ParseError: 
            raise ParseError("Invalid line: %r" % self.line)
      return self.sink

   def parsestring(self, s): 
      """Parse s as an N-Triples string."""
      if not isinstance(s, basestring): 
         raise ParseError("Item to parse must be a string instance.")
      from cStringIO import StringIO
      f = StringIO()
      f.write(s)
      f.seek(0)
      self.parse(f)

   def readline(self): 
      """Read an N-Triples line from buffered input."""
      # N-Triples lines end in either CRLF, CR, or LF
      # Therefore, we can't just use f.readline()
      if not self.buffer: 
         buffer = self.file.read(bufsiz)
         if not buffer: return None
         self.buffer = buffer

      while True: 
         m = r_line.match(self.buffer)
         if m: # the more likely prospect
            self.buffer = self.buffer[m.end():]
            return m.group(1)
         else: 
            buffer = self.file.read(bufsiz)
            if not buffer: 
               raise ParseError("EOF in line")
            self.buffer += buffer

   def parseline(self): 
      self.eat(r_wspace)
      if (not self.line) or self.line.startswith('#'): 
         return # The line is empty or a comment

      subject = self.subject()
      self.eat(r_wspaces)

      predicate = self.predicate()
      self.eat(r_wspaces)

      object = self.object()
      self.eat(r_tail)

      if self.line: 
         raise ParseError("Trailing garbage")
      self.sink.triple(subject, predicate, object)

   def peek(self, token): 
      return self.line.startswith(token)

   def eat(self, pattern): 
      m = pattern.match(self.line)
      if not m: # @@ Why can't we get the original pattern?
         raise ParseError("Failed to eat %s" % pattern)
      self.line = self.line[m.end():]
      return m

   def subject(self): 
      # @@ Consider using dictionary cases
      subj = self.uriref() or self.nodeid()
      if not subj: 
        raise ParseError("Subject must be uriref or nodeID")
      return subj

   def predicate(self): 
      pred = self.uriref()
      if not pred: 
         raise ParseError("Predicate must be uriref")
      return pred

   def object(self): 
      objt = self.uriref() or self.nodeid() or self.literal()
      if not objt: 
         raise ParseError("Unrecognised object type")
      return objt

   def uriref(self): 
      if self.peek('<'): 
         uri = self.eat(r_uriref).group(1)
         uri = unquote(uri)
         uri = uriquote(uri)
         return URI(uri)
      return False

   def nodeid(self): 
      if self.peek('_'): 
         return bNode(self.eat(r_nodeid).group(1))
      return False

   def literal(self): 
      if self.peek('"'): 
         lit, lang, dtype = self.eat(r_literal).groups()
         if lang and dtype: 
            raise ParseError("Can't have both a language and a datatype")
         lit = unquote(lit)
         return Literal(lit, lang, dtype)
      return False

def parseURI(uri): 
   import urllib
   parser = NTriplesParser()
   u = urllib.urlopen(uri)
   sink = parser.parse(u)
   u.close()
   # for triple in sink: 
   #    print triple
   print 'Length of input:', sink.length

def main(): 
   import sys
   if len(sys.argv) == 2: 
      parseURI(sys.argv[1])
   else: print __doc__

if __name__=="__main__": 
   main()



import os
from cStringIO import StringIO

from zope.interface import implements

try:
    import RDF
except ImportError:
    RDF = None

from sparrow.base_backend import BaseBackend
from sparrow.error import ConnectionError, TripleStoreError, QueryError
from sparrow.interfaces import ITripleStore, ISPARQLEndpoint
from sparrow.utils import (parse_sparql_result,
                           dict_to_ntriples,
                           ntriples_to_dict,
                           json_to_ntriples,
                           ntriples_to_json)

class RedlandTripleStore(BaseBackend):
    implements(ITripleStore, ISPARQLEndpoint)
    
    def __init__(self):
        self._nsmap = {}

    def connect(self, dburi):
        self._model = model_from_uri(dburi, contexts='yes')

    def disconnect(self):
        del self._model

    def contexts(self):
        return [c.literal_value['string'].decode('utf8') for c in
                self._model.get_contexts()]

    def register_prefix(self, prefix, namespace):
        self._nsmap[prefix] = namespace

    def add_rdfxml(self, data, context_name, base_uri):
        data = self._get_file(data)
        self._add_stream(self._parse(data, 'rdfxml', base_uri), context_name)
        
    def add_ntriples(self, data, context_name):
        data = self._get_file(data)
        self._add_stream(self._parse(data, 'ntriples', '-'), context_name)

    def add_turtle(self, data, context_name):
        data = self._get_file(data)
        self._add_stream(self._parse(data, 'turtle', '-'), context_name)

    def _add_stream(self, stream, context_name):
        if isinstance(context_name, unicode):
            context_name = context_name.encode('utf8')
        self._model.add_statements(stream, RDF.Node(context_name))
        
    def _parse(self, file, format, base_uri=None):            
        if format == 'turtle':
            parser = RDF.TurtleParser()
        else:
            parser = RDF.Parser(format)

        if isinstance(base_uri, unicode):
            base_uri = base_uri.encode('utf8')
        data = file.read()
        file.close()
        try:
            stream = parser.parse_string_as_stream(data, base_uri)
        except RDF.RedlandError, err:
            raise TripleStoreError(err)
        if stream is None:
            raise TripleStoreError('Parsing RDF Failed')
        return stream

    def get_rdfxml(self, context_name, pretty=False):
        if pretty:
            format = 'rdfxml-abbrev'
        else:
            format = 'rdfxml'
        return self._serialize(context_name, format)

    def get_ntriples(self, context_name):
        return self._serialize(context_name, 'ntriples')
        
    def get_turtle(self, context_name):
        return self._serialize(context_name, 'turtle')
    
    def _serialize(self, context_name, format):
        # this sucks, we need a temp model because contexts can not
        # be serialized :(

        if isinstance(context_name, unicode):
            context_name = context_name.encode('utf8')
            
        stream = self._model.as_stream(RDF.Node(context_name))            
        return self._serialize_stream(stream, format)
        
    def _serialize_stream(self, stream, format):
        temp = model_from_uri('memory')
        temp.add_statements(stream)
        serializer = RDF.Serializer(name=format)
        for prefix, ns in self._nsmap.items():
            serializer.set_namespace(prefix, ns)

        return StringIO(serializer.serialize_model_to_string(temp))

    def remove_rdfxml(self, data, context_name, base_uri):
        data = self._get_file(data)
        self._remove(data, 'rdfxml', context_name, base_uri)

    def remove_turtle(self, data, context_name):
        data = self._get_file(data)
        self._remove(data, 'turtle', context_name, '-')

    def remove_ntriples(self, data, context_name):
        data = self._get_file(data)
        self._remove(data, 'ntriples', context_name, '-')

    def _remove(self, file, format, context, base_uri):
        if isinstance(context, unicode):
            context = context.encode('utf8')
        context = RDF.Node(context)
        
        stream = self._parse(file, format, base_uri)
        for statement in stream:
            self._model.remove_statement(statement, context)
        
    def clear(self, context):
        if isinstance(context, unicode):
            context = context.encode('utf8')
        self._model.remove_statements_with_context(RDF.Node(context))
    
    def count(self, context=None):
        result = None
        if context is None:
            result = len(self._model)
        return result

    def _query(self, sparql):
        if isinstance(sparql, unicode):
            sparql = sparql.encode('utf8')
        query = RDF.SPARQLQuery(sparql)
        
        try:
            result = query.execute(self._model)
        except RDF.RedlandError, err:
            raise QueryError(err)
        return result

    def select(self, sparql):
        result = self._query(sparql)
        if not result.is_bindings():
            raise QueryError('SELECT Query did not return bindings')
        return parse_sparql_result(result.to_string())
        
    def ask(self, sparql):
        result = self._query(sparql)
        if not result.is_boolean():
            raise QueryError('ASK Query did not return a boolean')
        
        return result.get_boolean()

    def construct(self, sparql, format):
        out_format = format
        if format in ['json', 'dict']:
            out_format = 'ntriples'
        result = self._query(sparql)
        if not result.is_graph():
            raise QueryError('CONSTRUCT Query did not return a graph')
        
        m = RDF.Model()
        stream = result.as_stream()
        if stream is None:
            return
        result = self._serialize_stream(stream, out_format)
        if format == 'json':
            result = ntriples_to_json(result)
        elif format == 'dict':
            result = ntriples_to_dict(result)
        return result

    
def model_from_uri(uri=None, **opts):
    if RDF is None:
        raise ConnectionError(
            'Redland support is not available, install from librdf.org')
    
    options = []
    for key, val in opts.items():
        options.append("%s='%s'" % (key, val))
        
    options = ','.join(options)
    if uri == None or uri == 'memory':
        options += ",hash-type='memory'"
        store = RDF.Storage(storage_name='hashes',
                            name='memory_store',
                            options_string=options)
    elif uri.startswith('sqlite://'):
        file = uri[9:]
        try:
            store = RDF.Storage(storage_name='sqlite',
                                name=file,
                                options_string=options)
        except RDF.RedlandError:
            raise ConnectionError("Can't connect to %s" % uri)
    elif uri.startswith('bdb://'):
        file = uri[6:]
        if not os.path.isfile(file+'-sp2o.db') and not 'new=' in options:
            options += ",new='yes'"
        options += ",hash-type='bdb'"
        store = RDF.Storage(storage_name='hashes',
                            name=file,
                            options_string=options)
    elif uri.startswith('mysql://'):
        uri = uri[8:]
        userpass, hostdb = uri.split('@')
        user, pwd = userpass.split(':')
        host, db = hostdb.split('/')
        if ':' in host:
            host, port = host.split(':')
        else:
            port = '3306'
        if '#' in db:
            db, name = db.split('#')
        else:
            name = 'main'

        options += (",host='%s',port='%s',database='%s',user='%s',"
                    "password='%s'" % (host, port, db, user, pwd))
        try:
            store = RDF.Storage(storage_name='mysql',
                                name=name,
                                options_string=options)
        except RDF.RedlandError:
            raise ConnectionError("Can't connect to %s" % uri)
    else:
        raise ConnectionError('Unknown dburi: %s' % uri)
        
    return RDF.Model(store)

    



import os
from unittest import TestCase, TestSuite, makeSuite, main

import sparrow
from sparrow.error import ConnectionError
from sparrow.tests.base_tests import (TripleStoreTest,
                                      TripleStoreQueryTest,
                                      open_test_file)


# To run these tests, make sure the sesame.cfg buildout profile is used
# this will export a 'SESAME_HOST' and 'SESAME_PORT' variable to
# the environ dict which is used by this test to determine where
# the sesame server is running.

# The sesame server will need to be configured with a repository called
# 'test' this repository can be a simple in memory database.
# Run the configure_sesame tool to generate the test repository, and add the
# working repository as specified in the buildout profile

class SesameTest(TripleStoreTest):
    def setUp(self):
        self.db = sparrow.database('sesame', get_sesame_url())

    def tearDown(self):
        self.db.clear('test')
        self.db.disconnect()
        del self.db

class SesameQueryTest(TripleStoreQueryTest):
    def setUp(self):
        self.db = sparrow.database('sesame', get_sesame_url())
        fp = open_test_file('ntriples')
        self.db.add_ntriples(fp, 'test')
        fp.close()
        
    def tearDown(self):
        self.db.clear('test')
        self.db.disconnect()
        del self.db
    
def get_sesame_url():
    # host and port variables are set from
    # the buildout script (see profiles/sesame.cfg)
    url = 'http://%s:%s/test' % (os.environ.get('SESAME_HOST', 'localhost'),
                                 os.environ.get('SESAME_PORT', '8000'))
    return url

def test_suite():
    try:
        sparrow.database('sesame', get_sesame_url())
    except ConnectionError:
        # sesame not running?
        return TestSuite()

    suite = TestSuite()
    suite.addTest(makeSuite(SesameTest))
    suite.addTest(makeSuite(SesameQueryTest))
    return suite

if __name__ == '__main__':

    main(defaultTest='test_suite')



from unittest import TestCase, TestSuite, makeSuite, main
from StringIO import StringIO

from sparrow.utils import ntriples_to_dict, dict_to_ntriples

class DictFormatTest(TestCase):
    def test_uri_object(self):
        nt = '<uri:a> <uri:b> <uri:c> .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'uri:c',
                                    'type': u'uri'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_bnode_object(self):
        nt = '<uri:a> <uri:b> _:c .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'c',
                                    'type': u'bnode'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_bnode_subject(self):
        nt = '_:a <uri:b> _:c .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'_:a': {u'uri:b': [{'value': u'c',
                                  'type': u'bnode'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_literal_object(self):
        nt = '<uri:a> <uri:b> "foo" .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'foo',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_literal_language_object(self):
        nt = '<uri:a> <uri:b> "foo"@en .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'foo',
                                    'lang': u'en',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_literal_datatype_object(self):
        nt = '<uri:a> <uri:b> "foo"^^<uri:string> .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'foo',
                                    'datatype': u'uri:string',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_literal_value_quote_escape(self):
        nt = '<uri:a> <uri:b> "I say \\"Hello\\"." .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'I say "Hello".',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())
        
    def test_literal_value_backslash_escape(self):
        nt = '<uri:a> <uri:b> "c:\\\\temp\\\\foo.txt" .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'c:\\temp\\foo.txt',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    def test_literal_value_newline_tab_escape(self):
        nt = '<uri:a> <uri:b> "\\n\\tHello!\\n" .\n'
        data = ntriples_to_dict(StringIO(nt))
        self.assertEquals(
            data,
            {u'uri:a': {u'uri:b': [{'value': u'\n\tHello!\n',
                                    'type': u'literal'}]}})
        self.assertEquals(nt, dict_to_ntriples(data).read())

    
def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(DictFormatTest))
    return suite

if __name__ == '__main__':
    main(defaultTest='test_suite')




from sparrow.redland_backend import RedlandTripleStore
from sparrow.rdflib_backend import RDFLibTripleStore
from sparrow.sesame_backend import SesameTripleStore
from sparrow.allegro_backend import AllegroTripleStore

def database(backend, dburi):
    if backend == 'redland':
        db = RedlandTripleStore()
    elif backend == 'rdflib':
        db = RDFLibTripleStore()
    elif backend == 'sesame':
        db = SesameTripleStore()
    elif backend == 'allegro':
        db = AllegroTripleStore()
    else:
        raise ValueError('Unknown database backend: "%s"' % backend)
    db.connect(dburi)
    return db



import os
from os.path import join
import shutil
import subprocess
from cStringIO import StringIO
import socket
from urllib import quote, urlencode

import httplib2
from zope.interface import implements
from lxml import etree

from sparrow.base_backend import BaseBackend
from sparrow.error import ConnectionError, TripleStoreError, QueryError
from sparrow.interfaces import ITripleStore, ISPARQLEndpoint
from sparrow.utils import (parse_sparql_result,
                           dict_to_ntriples,
                           ntriples_to_dict,
                           json_to_ntriples,
                           ntriples_to_json)


class SesameTripleStore(BaseBackend):
    implements(ITripleStore, ISPARQLEndpoint)
    
    def __init__(self):
        self._nsmap = {}
        
    def connect(self, dburi):
        host, rest = dburi[7:].split(':', 1)
        port, self._name = rest.split('/', 1)
        self._url = 'http://%s:%s/openrdf-sesame' % (host, port)
        self._http = httplib2.Http()
        try:
            resp, content = self._http.request(
                '%s/repositories' % self._url,
                "GET",
                headers={'Accept': 'application/sparql-results+xml'})
        except socket.error:
            raise ConnectionError(
                'Can not connect to repository, is it running?')
        
        if resp['status'] != '200':
            raise ConnectionError(
                'Can not connect to server: %s' % resp['status'])
            
        for repo in parse_sparql_result(content):
            if repo['id']['value'] == self._name:
                break
        else:
            raise ConnectionError(
                'Server has no repository: %s' % self._name)
        
    def disconnect(self):
        del self._http

    def contexts(self):
        resp, content = self._http.request(
            '%s/repositories/%s/contexts' % (self._url, self._name),
            "GET",
            headers={'Accept': 'application/sparql-results+xml'})

        return [c['contextID']['value'].split(':', 1)[1] for c in
                parse_sparql_result(content)]

    def _get_mimetype(self, format):
        return {'ntriples': 'text/plain',
                'rdfxml': 'application/rdf+xml',
                'turtle': 'application/x-turtle',
                'n3': 'text/rdf+n3',
                'trix': 'application/trix',
                'trig': 'applcation/x-trig'}[format]

    def _get_context(self, context):
        return '<context:%s>' % context
        
    def register_prefix(self, prefix, namespace):
        # store also in _nsmap for allegro turtle workaround
        self._nsmap[prefix] = namespace
        
        clength = str(len(namespace))
        resp, content = self._http.request(
            '%s/repositories/%s/namespaces/%s' % (
                self._url, self._name, prefix),
            'PUT',
            body=namespace,
            headers = {"Content-length": clength})
        if resp['status'] != '204':
            raise TripleStoreError(content)

    def add_rdfxml(self, data, context, base_uri):
        data = self._get_file(data)
        self._add(data, 'rdfxml', context, base_uri)

    def add_ntriples(self, data, context):
        data = self._get_file(data)
        self._add(data, 'ntriples', context)

    def add_turtle(self, data, context):
        data = self._get_file(data)
        self._add(data, 'turtle', context)
        
    def _add(self, file, format, context, base_uri=None):
        data = file.read()
        file.close()
        clength = str(len(data))
        ctype = self._get_mimetype(format)
        params = {'context': self._get_context(context)}
        if base_uri:
            params['baseURI'] = '<%s>' % base_uri
        params = urlencode(params)
        
        resp, content = self._http.request(
            '%s/repositories/%s/statements?%s' % (
                self._url, self._name, params),
            'POST',
            body=data,
            headers = {"Content-type": ctype,
                       "Content-length": clength})
        
        if resp['status'] != '204':
            raise TripleStoreError(content)       

    def get_rdfxml(self, context):
        return self._serialize('rdfxml', context)

    def get_turtle(self, context):
        return self._serialize('turtle', context)

    def get_ntriples(self, context):
        return self._serialize('ntriples', context)
        
    def _serialize(self, format, context, pretty=False):
        
        context = quote(self._get_context(context))
        ctype = self._get_mimetype(format)
        
        resp, content = self._http.request(
            '%s/repositories/%s/statements?context=%s' % (
                self._url, self._name, context),
            'GET',
            headers = {"Accept": ctype})
        if resp['status'] != '200':
            raise TripleStoreError(content)

        return StringIO(content)

    def remove_rdfxml(self, data, context, base_uri):
        data = self._get_file(data)
        self._remove(data, 'rdfxml', context, base_uri)

    def remove_turtle(self, data, context):
        data = self._get_file(data)
        self._remove(data, 'turtle', context)

    def remove_ntriples(self, data, context):
        data = self._get_file(data)
        self._remove(data, 'ntriples', context)
        
    def _remove(self, file, format, context, base_uri=None):
        data = file.read()
        file.close()
        clength = str(len(data))
        ctype = self._get_mimetype(format)
        params = {'context': self._get_context(context)}
        if base_uri:
            params['baseURI'] = '<%s>' % base_uri
        params = urlencode(params)
        resp, content = self._http.request(
            '%s/repositories/%s/statements?%s' % (
                self._url, self._name, params),
            'DELETE',
            body=data,
            headers = {"Content-type": ctype,
                       "Content-length": clength})
        if resp['status'] != '204':
            raise TripleStoreError(content)
    
    def clear(self, context):
        context = quote(self._get_context(context))
        resp, content = self._http.request(
            '%s/repositories/%s/statements?context=%s' % (
                self._url, self._name, context),
            'DELETE')
        if resp['status'] != '204':
            raise TripleStoreError(content)
        
    def count(self, context=None):
        if context is None:
            context = ''
        else:
            context = '?context='+quote(self._get_context(context))
        resp, content = self._http.request(
            '%s/repositories/%s/size%s' % (
                self._url, self._name, context),
            'GET')
        if resp['status'] != '200':
            raise TripleStoreError(content)
        return int(content)
    
    def select(self, sparql):
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': 'application/sparql-results+xml'})
        
        if resp['status'] != '200':
            raise QueryError(content)

        # Allegro Graph returns status 200 when parsing failed
        if content.startswith('Server error:'):
            raise QueryError(content[14:])
        
        return parse_sparql_result(content)
    
    def ask(self, sparql):
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': 'application/sparql-results+xml'})

        if resp['status'] != '200':
            raise QueryError(content)
        # Allegro Graph returns status 200 when parsing failed
        if content.startswith('Server error:'):
            raise QueryError(content[14:])
        
        return parse_sparql_result(content)
    
    def construct(self, sparql, format):
        out_format = format
        if format in ['json', 'dict']:
            out_format = 'ntriples'
        ctype = self._get_mimetype(out_format)        
        params = urlencode({'query': sparql,
                            'queryLn': 'SPARQL',
                            'infer':'false'})
        
        resp, content = self._http.request(
            '%s/repositories/%s?%s' % (
            self._url, self._name, params),
            'GET',
            headers={'Accept': ctype})

        if resp['status'] != '200':
            raise QueryError(content)
        # Allegro Graph returns status 200 when parsing failed
        if content.startswith('Server error:'):
            raise QueryError(content[14:])

        result = StringIO(content)
        if format == 'json':
            result = ntriples_to_json(result)
        elif format == 'dict':
            result = ntriples_to_dict(result)
        return result

def start_server(host, port, uri, id, title, path):
    tomcat_dir = join(path ,'parts', 'tomcat-install')

    # check if port and host are configured in tomcats server.xml
    server_conf = join(tomcat_dir, 'conf', 'server.xml')
    doc = etree.parse(server_conf)
    connector_el = doc.xpath('//Service[@name="Catalina"]/Connector')[0]
    if int(connector_el.attrib['port']) != port:
        connector_el.attrib['port'] = str(port)
        doc.write(server_conf)
    engine_el = doc.xpath(
        '//Service[@name="Catalina"]/Engine[@name="Catalina"]')[0]
    if engine_el.attrib['defaultHost'] != host:
        engine_el.attrib['defaultHost'] = host

        host_el = engine_el.xpath('Host')[0]
        if host_el.attrib['name'] != host:
            host_el.attrib['name'] = host
        doc.write(server_conf)
    
    # copy the war files to tomcat
    sesame_dir = join(path , 'parts', 'sesame-install')
    if not os.path.isfile(join(tomcat_dir, 'webapps', 'openrdf-sesame.war')):
        shutil.copyfile(join(sesame_dir, 'war', 'openrdf-sesame.war'),
                        join(tomcat_dir, 'webapps', 'openrdf-sesame.war'))
    if not os.path.isfile(join(tomcat_dir, 'webapps', 'openrdf-workbench.war')):
        shutil.copyfile(join(sesame_dir, 'war', 'openrdf-workbench.war'),
                        join(tomcat_dir, 'webapps', 'openrdf-workbench.war'))
    
    
    catalina = join(tomcat_dir ,'bin', 'catalina.sh')
    
    os.system('%s run' % catalina)


SESAME_TEST_TEMPLATE = """
drop test.
yes
create memory.
test
Test Repository
false
0
"""

SESAME_MEMORY_TEMPLATE = """
create memory.
%(id)s
%(title)s
false
0
"""

SESAME_NATIVE_TEMPLATE = """
create native.
%(id)s
%(title)s
spoc, posc, opsc
"""

SESAME_MYSQL_TEMPLATE = """
create mysql.
%(id)s
%(title)s
com.mysql.jdbc.Driver
%(host)s
%(port)s
%(db)s
0
%(user)s
%(pwd)s
256
"""

def configure_server(host, port, uri, id, title, path):

    variables = {'id': id, 'title':title}
    
    if uri == 'memory':
        template = SESAME_MEMORY_TEMPLATE % variables
    elif uri == 'native':
        template = SESAME_NATIVE_TEMPLATE % variables
    elif uri.startswith('mysql://'):
        uri = uri[8:]
        userpart, dbpart = uri.split('@')
        user, pwd = userpart.split(':')
        host, db = dbpart.split('/', 1)
        if ':' in host:
            host, port = host.split(':', 1)
        else:
            port = '3306'
        variables['user'] = user
        variables['pwd'] = pwd
        variables['db'] = db
        variables['host'] = host
        variables['port'] = port
        template = SESAME_MYSQL_TEMPLATE % variables
    else:
        raise ValueError('Unknown Sesame backend URI: %s' % uri)
        
    conf_script = join(path, 'parts', 'sesame-install', 'bin', 'console.sh')
    proc = subprocess.Popen(conf_script, shell=True, stdin=subprocess.PIPE)
    # connect to sesame
    proc.communicate("connect http://%s:%s/openrdf-sesame." % (host, port)+
                     SESAME_TEST_TEMPLATE +
                     template)



import os
from unittest import TestCase, TestSuite, makeSuite, main

import sparrow
from sparrow.error import ConnectionError
from sparrow.tests.base_tests import (TripleStoreTest,
                                      TripleStoreQueryTest,
                                      open_test_file)


# To run these tests, make sure the allegro.cfg buildout profile is used
# this will export a 'ALLEGRO_HOST' and 'ALLEGRO_PORT' variable to
# the environ dict which is used by this test to determine where
# the allegro server is running.

# The allegro server will need to be configured with a repository called
# 'test' this repository can be a simple in memory database.
# Run the configure_allegro tool to generate the test repository, and add the
# working repository as specified in the buildout profile

# Note that the state of the database can get messed up if tests fail or
# the tests are cancelled. You can clear the whole database with the following
# command:
# curl -X DELETE "http://localhost:8001/sesame/repositories/test/statements"
#
# If you get connection errors, you might want to create a directory 'test'
# in the directory where you started the allegro server

class AllegroTest(TripleStoreTest):
    def setUp(self):
        self.db = sparrow.database('allegro', get_allegro_url())

    def tearDown(self):
        self.db.clear('test')
        self.db.disconnect()
        del self.db

class AllegroQueryTest(TripleStoreQueryTest):
    def setUp(self):
        self.db = sparrow.database('allegro', get_allegro_url())
        fp = open_test_file('ntriples')
        self.db.add_ntriples(fp, 'test')
        fp.close()

    def tearDown(self):
        self.db.clear('test')
        self.db.disconnect()
        del self.db
    
def get_allegro_url():
    # host and port variables are set from
    # the buildout script (see profiles/allegro.cfg)
    url = 'http://%s:%s/test' % (os.environ.get('ALLEGRO_HOST', 'localhost'),
                                 os.environ.get('ALLEGRO_PORT', '8000'))
    
    return url

def test_suite():
    try:
        sparrow.database('allegro', get_allegro_url())
    except ConnectionError, err:
        # allegro not running?
        return TestSuite()

    suite = TestSuite()
    suite.addTest(makeSuite(AllegroTest))
    suite.addTest(makeSuite(AllegroQueryTest))
    return suite

if __name__ == '__main__':

    main(defaultTest='test_suite')



import os
import StringIO
from unittest import TestCase, TestSuite, makeSuite, main

from sparrow.interfaces import ITripleStore, ISPARQLEndpoint
from sparrow.error import TripleStoreError, QueryError

TESTFILE = 'wine'
FORMATS = ['ntriples', 'rdfxml', 'turtle', 'json']

def open_test_file(format):
    extension = {'rdfxml': '.rdf',
                 'ntriples': '.nt',
                 'turtle': '.ttl',
                 'json': '.json'}
    filename = 'wine' + extension[format]
    return open(
        os.path.join(os.path.dirname(__file__), filename), 'r')

class TripleStoreTest(TestCase):

    def test_broken_parsing(self):
        self.assertRaises(
            TripleStoreError,
            self.db.add_rdfxml,
            '@', 'test', 'http://example.org')
        self.assertRaises(
            TripleStoreError,
            self.db.add_ntriples,
            '@', 'test')
        self.assertRaises(
            TripleStoreError,
            self.db.add_turtle,
            '@', 'test')
        self.assertRaises(
            TripleStoreError,
            self.db.add_json,
            '@', 'test')

    def test_rdfxml_parsing(self):
        self.db.add_rdfxml(open_test_file('rdfxml'), 'test', 'file://wine.rdf')
        triples = self.db.count('test') or self.db.count()
        if not triples is None:
            self.assertTrue(triples > 1500)

    def test_ntriples_parsing(self):
        self.db.add_ntriples(open_test_file('ntriples'), 'test')
        triples = self.db.count('test') or self.db.count()
        if not triples is None:
            self.assertTrue(triples > 1500)

    def test_turtle_parsing(self):
        self.db.add_turtle(open_test_file('turtle'), 'test')
        triples = self.db.count('test') or self.db.count()
        if not triples is None:
            self.assertTrue(triples > 1500)

    def test_json_parsing(self):
        self.db.add_json(open_test_file('json'), 'test')
        triples = self.db.count('test') or self.db.count()
        if not triples is None:
            self.assertTrue(triples > 1500)

    def test_base_uri_parsing(self):
        data = StringIO.StringIO('''<?xml version="1.0"?>
<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
 <rdf:Description rdf:about="">
  <foo:name xmlns:foo="http://foobar.com#">bar</foo:name>
 </rdf:Description>
</rdf:RDF>''')
        self.db.add_rdfxml(data, 'test', 'http://example.org')
        result = self.db.get_ntriples('test').read()
        self.assertEquals(result.strip()[:-1].strip(),
                          '<http://example.org> <http://foobar.com#name> "bar"')
        
    def test_remove_statements(self):
        self.db.add_ntriples(open_test_file('ntriples'), 'test')
        fp = StringIO.StringIO(
            '<http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine> '
            '<http://www.w3.org/2000/01/rdf-schema#label> '
            '"Wine Ontology" .\n')
        
        self.db.remove_ntriples(fp, 'test')
        data = self.db.get_ntriples('test').read()
        self.assertTrue('Wine Ontology' not in data)

    def test_ntriples_serializing(self):
        self.db.add_ntriples(open_test_file('ntriples'), 'test')
        data = self.db.get_ntriples('test').read()
        self.assertTrue('Wine Ontology' in data)

    def test_rdfxml_serializing(self):
        self.db.add_rdfxml(open_test_file('rdfxml'), 'test', 'file://wine.rdf')
        self.db.register_prefix(
            'vin',
            'http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#')
        data = self.db.get_rdfxml('test').read()
        self.assertTrue('Wine Ontology' in data)
        self.assertTrue('xmlns:vin' in data)

    def test_turtle_serializing(self):
        self.db.add_turtle(open_test_file('turtle'), 'test')
        self.db.register_prefix(
            'vin',
            'http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#')
        data = self.db.get_turtle('test').read()
        self.assertTrue('Wine Ontology' in data)
        self.assertTrue('@prefix vin' in data)
        
    def test_clear(self):
        count = self.db.count('test')
        if count is None:
            return
        self.assertEquals(count , 0)
        self.db.add_ntriples(open_test_file('ntriples'), 'test')
        count = self.db.count()
        self.assertTrue(count > 1500)
        self.db.clear('test')
        count = self.db.count()
        self.assertEquals(count , 0)

    def test_contexts(self):
        self.assertEquals(list(self.db.contexts()), [])
        self.db.add_ntriples(open_test_file('ntriples'), 'a')
        self.db.add_ntriples(open_test_file('ntriples'), 'b')
        self.db.add_ntriples(open_test_file('ntriples'), 'c')
        self.assertEquals(sorted(list(self.db.contexts())), [u'a', u'b', u'c'])
        self.db.clear('b')
        self.assertEquals(sorted(list(self.db.contexts())), [u'a', u'c'])
        self.db.clear('a')
        self.db.clear('c')
        self.assertEquals(list(self.db.contexts()), [])
        
    
class TripleStoreQueryTest(TestCase):
    def test_ask(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        ASK { ?x rdfs:label "Wine Ontology"}
        """
        self.assertTrue(self.db.ask(q))
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        ASK { ?x rdfs:label "FooBar"}
        """
        self.assertFalse(self.db.ask(q))

    def test_select(self):
        q = """
        prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        prefix vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        select ?grape
        where { ?grape a vin:WineGrape .}
        """
        result = self.db.select(q)
        self.assertEquals(len(result), 16)
        grapes = sorted(
            [r['grape']['value'].split('#')[-1] for r in result])
        
        self.assertEquals(grapes, [
            u'CabernetFrancGrape', u'CabernetSauvignonGrape',
            u'ChardonnayGrape', u'CheninBlancGrape', u'GamayGrape',
            u'MalbecGrape', u'MerlotGrape', u'PetiteSyrahGrape',
            u'PetiteVerdotGrape', u'PinotBlancGrape', u'PinotNoirGrape',
            u'RieslingGrape', u'SangioveseGrape', u'SauvignonBlancGrape',
            u'SemillonGrape', u'ZinfandelGrape'])

    def test_select_literal_language(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        SELECT ?label
        WHERE { vin:Wine rdfs:label ?label .}
        """
        result = self.db.select(q)
        self.assertEquals(sorted(result),
                           [{u'label': {'type': u'literal',
                                        'value': u'wine',
                                        'lang': u'en'}},
                            {u'label': {'type': u'literal',
                                        'value': u'vin',
                                        'lang': u'fr'}}])

    def test_select_literal_datatype(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX vin: <http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine#>
        SELECT ?year
        WHERE { vin:Year1998 vin:yearValue ?year .}
        """
        result = self.db.select(q)
        self.assertEquals(
            sorted(result),
            [{u'year': {
            'type': u'literal',
            'value': u'1998',
            'datatype': u'http://www.w3.org/2001/XMLSchema#positiveInteger'}}])
        
    def test_construct(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        CONSTRUCT {?x rdfs:label "Wine Ontology"}
        WHERE { ?x rdfs:label "Wine Ontology" .}
        """
        fp = self.db.construct(q, 'ntriples')
        self.assertEquals(
            fp.read().strip()[:-1].strip(),
            ('<http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine> '
             '<http://www.w3.org/2000/01/rdf-schema#label> "Wine Ontology"'))


        fp.close()
        
    def test_construct_dict(self):
        q = """
        PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>        
        CONSTRUCT {?x rdfs:label "Wine Ontology"}
        WHERE { ?x rdfs:label "Wine Ontology" .}
        """
        data = self.db.construct(q, 'dict')
        self.assertEquals(data, {
            u'http://www.w3.org/TR/2003/PR-owl-guide-20031209/wine': {
            u'http://www.w3.org/2000/01/rdf-schema#label': [
            {'type': u'literal', 'value': u'Wine Ontology'}]}})
        
    def test_broken_query(self):
        self.assertRaises(QueryError,
                          self.db.select,
                          'foo')
        self.assertRaises(QueryError,
                          self.db.ask,
                          'foo')
        self.assertRaises(QueryError,
                          self.db.construct,
                          'foo', 'rdfxml')
        



from unittest import TestCase, TestSuite, makeSuite, main

import sparrow
from sparrow.error import ConnectionError
from sparrow.tests.base_tests import (TripleStoreTest,
                                      TripleStoreQueryTest,
                                      open_test_file)

class RedlandTest(TripleStoreTest):
    def setUp(self):
        self.db = sparrow.database('redland', 'memory')

    def tearDown(self):
        self.db.disconnect()
        del self.db

class RedlandQueryTest(TripleStoreQueryTest):
    def setUp(self):
        self.db = sparrow.database('redland', 'memory')
        self.db.add_ntriples(open_test_file('ntriples'), 'test')
        
    def tearDown(self):
        self.db.disconnect()
        del self.db

def test_suite():
    try:
        sparrow.database('redland', 'memory')
    except ConnectionError:
        # redland / librdf not installed?
        return TestSuite()
    suite = TestSuite()
    suite.addTest(makeSuite(RedlandTest))
    suite.addTest(makeSuite(RedlandQueryTest))
    return suite

if __name__ == '__main__':
    main(defaultTest='test_suite')



from cStringIO import StringIO

from zope.interface import implements

try:
    import rdflib
    from rdflib.Graph import Graph, ConjunctiveGraph
    from rdflib.store.IOMemory import IOMemory
except ImportError:
    rdflib = None

from sparrow.base_backend import BaseBackend
from sparrow.error import ConnectionError, TripleStoreError, QueryError
from sparrow.interfaces import ITripleStore, ISPARQLEndpoint
from sparrow.utils import (parse_sparql_result,
                           dict_to_ntriples,
                           ntriples_to_dict,
                           json_to_ntriples,
                           ntriples_to_json)


class RDFLibTripleStore(BaseBackend):
    implements(ITripleStore, ISPARQLEndpoint)

    def __init__(self):
        self._nsmap = {}

    def connect(self, dburi):
        if rdflib is None:
            raise ConnectionError('RDFLib backend is not installed')
        if dburi == 'memory':
            self._store = IOMemory()
        else:
            raise ConnectionError('Unknown database config: %s' % dburi)
    
    def disconnect(self):
        pass
        
    def _rdflib_format(self, format):
        return {'ntriples': 'nt',
                'rdfxml': 'xml',
                'turtle': 'n3'}[format]

    def contexts(self):
        return [c._Graph__identifier.decode('utf8') for c in
                self._store.contexts()]

    def _get_context(self, context_name):
        for ctxt in self._store.contexts():
            if ctxt._Graph__identifier == context_name:
                return ctxt
    
    def register_prefix(self, prefix, namespace):
        self._nsmap[prefix] = namespace


    def _parse(self, graph, file, format, base_uri=None):
        try:
            graph.parse(file, base_uri, format)
        except rdflib.exceptions.ParserError, err:
            # not sure if this ever happens
            raise TripleStoreError(err)
        except Exception, err:
            # each parser throws different errors,
            # there's an ntriples error, but the rdfxml
            # parser throws no errors so you end up with
            # a saxparser exception.
            # The n3 parser just silently fails
            # without any traceback
            raise TripleStoreError(err)
        
    def add_rdfxml(self, data, context, base_uri):
        data = self._get_file(data)
        graph = Graph(self._store, identifier=context)
        self._parse(graph, data, 'xml', base_uri)
        
    def add_ntriples(self, data, context):
        data = self._get_file(data)
        graph = Graph(self._store, identifier=context)
        self._parse(graph, data, 'nt')

    def add_turtle(self, data, context):
        data = self._get_file(data)
        graph = Graph(self._store, identifier=context)
        self._parse(graph, data, 'n3')

    def _serialize(self, graph, format, pretty=False):        
        for prefix, namespace in self._nsmap.items():
            graph.bind(prefix, namespace)
        return StringIO(graph.serialize(format=format))
        return result

    def get_rdfxml(self, context, pretty=False):
        return self._serialize(self._get_context(context), 'xml')

    def get_turtle(self, context):
        return self._serialize(self._get_context(context), 'n3')

    def get_ntriples(self, context):
        return self._serialize(self._get_context(context), 'nt')
    
    def remove_rdfxml(self, data, context, base_uri):    
        data = self._get_file(data)
        self._remove(data, context, 'xml', base_uri)

    def remove_ntriples(self, data, context):
        data = self._get_file(data)
        self._remove(data, context, 'nt')

    def remove_turtle(self, data, context):
        data = self._get_file(data)
        self._remove(data, context, 'n3')

    def _remove(self, file, context, format, base_uri=None):
        graph = Graph()
        self._parse(graph, file, format=format, base_uri=base_uri)
        context = self._get_context(context)        
        for triple in graph:
            self._store.remove(triple, context)
        
    def clear(self, context):
        context = self._get_context(context)
        self._store.remove((None, None, None), context)
        
    def count(self, context=None):
        context = self._get_context(context)
        
        if not context is None:
            return len(context)
        
        return len(self._store)

    def _query(self, sparql):
        try:
            result = ConjunctiveGraph(self._store).query(sparql)
        except SyntaxError, err:
            raise QueryError(err)
        return result
        
    def select(self, sparql):
        result = self._query(sparql)
        return parse_sparql_result(result.serialize())
        
    def ask(self, sparql):
        result = self._query(sparql)
        return result.askAnswer[0]
    
    def construct(self, sparql, format):
        out_format = format
        if format in ['json', 'dict']:
            out_format = 'ntriples'
            
        result = self._query(sparql)
        if not result.construct:
            raise QueryError('CONSTRUCT Query did not return a graph')

        result = self._serialize(result.result, self._rdflib_format(out_format))
        if format == 'json':
            result = ntriples_to_json(result)
        elif format == 'dict':
            result = ntriples_to_dict(result)
        return result



from cStringIO import StringIO
import urllib2

from sparrow.utils import (json_to_ntriples,
                           dict_to_ntriples,
                           ntriples_to_json,
                           ntriples_to_dict)
from sparrow.error import TripleStoreError

class BaseBackend(object):


    def _is_uri(self, data):
        if not isinstance(data, basestring):
            return False
        return data.startswith('http://') or data.startswith('file://')

    def _get_file(self, data):
        if self._is_uri(data):
            if data.startswith('file://'):
                return open(data[7:], 'r')
            elif data.startswith('http://'):
                return urllib2.urlopen(data)
        elif (hasattr(data, 'read') and
              hasattr(data, 'seek') and
              hasattr(data, 'close')):
            return data
        
        return StringIO(data)

    def add_json(self, data, context_name):
        data = self._get_file(data)
        try:
            data = json_to_ntriples(data)
        except ValueError, err:
            raise TripleStoreError(err)
        
        self.add_ntriples(data, context_name)

    def add_dict(self, data, context_name):
        data = dict_to_ntriples(data)
        self.add_ntriples(data, context_name)

    def get_json(self, context_name):
        data = self.get_ntriples(context_name)
        return ntriples_to_json(data)

    def get_dict(self, context_name):
        data = self.get_ntriples(context_name)
        return ntriples_to_dict(data)

    def remove_json(self, data, context_name):
        data = self._get_file(data)
        data = json_to_ntriples(data)
        self.remove_ntriples(data, context_name)

    def remove_dict(self, data, context_name):
        data = dict_to_ntriples(data)
        self.remove_ntriples(data, context_name)



import os, sys
from os.path import join
from urllib import quote, urlencode
import socket
from StringIO import StringIO
import httplib2
from zope.interface import implements

from sparrow.error import ConnectionError, TripleStoreError, QueryError
from sparrow.interfaces import ITripleStore, ISPARQLEndpoint
from sparrow.utils import (parse_sparql_result,
                           ntriples_to_json,
                           ntriples_to_dict)

from sparrow.sesame_backend import SesameTripleStore

class AllegroTripleStore(SesameTripleStore):
    def connect(self, dburi):
        host, rest = dburi[7:].split(':', 1)
        port, self._name = rest.split('/', 1)
        
        self._url = 'http://%s:%s/sesame' % (host, port)
        self._http = httplib2.Http()
        params = urlencode({'id': self._name,
                            'if-exists': 'open'})
        try:
            resp, content = self._http.request(
                '%s/repositories?%s' % (self._url, params),
                "POST")
        except socket.error:
            raise ConnectionError(
                'Can not connect to repository, is it running?')
        
        if resp['status'] != '204':
            raise ConnectionError(
                'Can not connect to server: %s' % resp['status'])
        
    def _turtle_to_ntriples(self, data):
        # Turtle syntax is not supported by allegro graph
        # HACK workaround using redland
        import RDF
        model = RDF.Model()
        parser = RDF.TurtleParser()
        try:
            parser.parse_string_into_model(model, data.read(),'-')
        except RDF.RedlandError, err:
            raise TripleStoreError(err)
        
        serializer = RDF.Serializer(name='ntriples')
        return StringIO(serializer.serialize_model_to_string(model))
        
    def _rdfxml_to_ntriples(self, data):
        # Ntriples syntax is not supported by allegro graph
        # as a result format for SPARQL Construct Queries
        # HACK workaround using redland
        import RDF
        model = RDF.Model()
        parser = RDF.Parser()
        try:
            parser.parse_string_into_model(model, data.read(),'-')
        except RDF.RedlandError, err:
            raise TripleStoreError(err)
        
        serializer = RDF.Serializer(name='ntriples')
        return StringIO(serializer.serialize_model_to_string(model))
        
    def _ntriples_to_turtle(self, data):
        # Turtle syntax is not supported by allegro graph
        # HACK workaround using redland
        import RDF
        model = RDF.Model()
        parser = RDF.Parser('ntriples')
        data = data.read()
        data = (data.strip() + '\n')
        try:
            parser.parse_string_into_model(model, data,'-')
        except RDF.RedlandError, err:
            raise TripleStoreError(err)
        
        serializer = RDF.Serializer(name='turtle')
        for prefix, ns in self._nsmap.items():
            serializer.set_namespace(prefix, ns)
        return StringIO(serializer.serialize_model_to_string(model))

    def add_turtle(self, data, context):
        data = self._get_file(data)
        data = self._turtle_to_ntriples(data)
        self.add_ntriples(data, context)
        
    def remove_turtle(self, data, context):
        data = self._get_file(data)
        data = self._turtle_to_ntriples(data)
        self.remove_ntriples(data, context)

    def get_turtle(self, context):
        data = self.get_ntriples(context)
        return self._ntriples_to_turtle(data)

    def construct(self, query, format):
        result =  super(AllegroTripleStore, self).construct(query, 'rdfxml')
        if format == 'rdfxml':
            return result
        
        result = self._rdfxml_to_ntriples(result)
        if format == 'ntriples':
            return result
        elif format == 'json':
            return  ntriples_to_json(result)
        elif format == 'dict':
            return ntriples_to_dict(result)
        elif format == 'turtle':
            return self._ntriples_to_turtle(result)



def start_server(host, port, path):
    part_dir = join(path ,'parts', 'allegro-install')
    allegro_dir = join(part_dir,
                       [d for d in os.listdir(part_dir) if
                        d.startswith('agraph')][0])

    allegro_cmd = join(allegro_dir, 'AllegroGraphServer')
    os.system('%s --http-port %s' % (allegro_cmd, port))
    fp = open(join(allegro_dir, 'agraph.pid'), 'r')
    pid = fp.read().strip()
    fp.close()

    # create storage directory for test repository
    if not os.path.isdir('test'):
        os.mkdir('test')
    
    print >> sys.stderr, 'Started Allegro server on port %s (pid %s)' % (
        port, pid)
    
    



from setuptools import setup, find_packages
from os.path import join, dirname

README = join(dirname(__file__), 'src', 'sparrow', 'README.txt')

setup(
    name='sparrow',
    version='1.0b4',
    author='Jasper Op de Coul (Infrae)',
    author_email='jasper@infrae.com',
    description="Sparrow, Common RDF/SPARQL Database API",
    long_description=open(README).read() + open('HISTORY.txt').read(),
    classifiers=["Development Status :: 4 - Beta",
                 "Programming Language :: Python",
                 "License :: OSI Approved :: BSD License",
                 "Topic :: Software Development :: Libraries :: Python Modules",
                 "Environment :: Web Environment"],
    keywords='python RDF SPARQL',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe=False,
    license='BSD',
    entry_points= {
    'console_scripts': [
        'start_sesame_server = sparrow.sesame_backend:start_server',
        'configure_sesame = sparrow.sesame_backend:configure_server',
        'start_allegro_server = sparrow.allegro_backend:start_server'
      ]
    },
    install_requires=[
    'zope.interface',
    'simplejson',
    'httplib2',
    'lxml',
    ],
)
